@@ -95,8 +95,9 @@ enum {
 
 #define R_ANAL_ARCHINFO_MIN_OP_SIZE 0
 #define R_ANAL_ARCHINFO_MAX_OP_SIZE 1
-#define R_ANAL_ARCHINFO_ALIGN 2
-#define R_ANAL_ARCHINFO_DATA_ALIGN 4
+#define R_ANAL_ARCHINFO_INV_OP_SIZE 2
+#define R_ANAL_ARCHINFO_ALIGN 4
+#define R_ANAL_ARCHINFO_DATA_ALIGN 8
 
 /* copypaste from r_asm.h */
 
@@ -586,7 +587,6 @@ typedef struct r_anal_options_t {
 	int bb_max_size;
 	bool trycatch;
 	bool norevisit;
-	int afterjmp; // continue analysis after jmp eax or forward jmp // option
 	int recont; // continue on recurse analysis mode
 	int noncode;
 	int nopskip; // skip nops at the beginning of functions
@@ -684,6 +684,7 @@ typedef struct r_anal_t {
 	SetU *visited;
 	RStrConstPool constpool;
 	RList *leaddrs;
+	char *pincmd;
 } RAnal;
 
 typedef enum r_anal_addr_hint_type_t {
@@ -748,15 +749,6 @@ typedef struct r_anal_hint_t {
 	ut64 stackframe;
 } RAnalHint;
 
-typedef RAnalFunction *(* RAnalGetFcnIn)(RAnal *anal, ut64 addr, int type);
-typedef RAnalHint *(* RAnalGetHint)(RAnal *anal, ut64 addr);
-
-typedef struct r_anal_bind_t {
-	RAnal *anal;
-	RAnalGetFcnIn get_fcn_in;
-	RAnalGetHint get_hint;
-} RAnalBind;
-
 typedef const char *(*RAnalLabelAt) (RAnalFunction *fcn, ut64);
 
 typedef enum {
@@ -897,6 +889,21 @@ typedef struct r_anal_op_t {
 	RAnalDataType datatype;
 } RAnalOp;
 
+typedef RAnalFunction *(* RAnalGetFcnIn)(RAnal *anal, ut64 addr, int type);
+typedef RAnalHint *(* RAnalGetHint)(RAnal *anal, ut64 addr);
+typedef int (* RAnalDecode)(RAnal *anal, RAnalOp *op, ut64 addr, const ut8 *data, int len, RAnalOpMask mask);
+typedef void (* RAnalOpInit)(RAnalOp *op);
+typedef void (* RAnalOpFini)(RAnalOp *op);
+
+typedef struct r_anal_bind_t {
+	RAnal *anal;
+	RAnalGetFcnIn get_fcn_in;
+	RAnalGetHint get_hint;
+	RAnalDecode decode;
+	RAnalOpInit opinit;
+	RAnalOpFini opfini;
+} RAnalBind;
+
 #define R_ANAL_COND_SINGLE(x) (!x->arg[1] || x->arg[0]==x->arg[1])
 
 typedef struct r_anal_cond_t {
@@ -1141,7 +1148,6 @@ typedef struct r_anal_esil_t {
 	int nowrite;
 	int iotrap;
 	int exectrap;
-	int repeat;
 	int parse_stop;
 	int parse_goto;
 	int parse_goto_count;
@@ -1174,6 +1180,7 @@ typedef struct r_anal_esil_t {
 	RAnalEsilTrace *trace;
 	RAnalEsilCallbacks cb;
 	RAnalReil *Reil;
+	char *pending; // pending op computed as a macro
 	// this is so cursed, can we please remove external commands from esil internals.
 	// Function pointers are fine, but not commands
 	char *cmd_step; // r2 (external) command to run before a step is performed
@@ -1531,7 +1538,7 @@ R_API int r_anal_add(RAnal *anal, RAnalPlugin *foo);
 R_API int r_anal_archinfo(RAnal *anal, int query);
 R_API bool r_anal_use(RAnal *anal, const char *name);
 R_API bool r_anal_esil_use(RAnal *anal, const char *name);
-R_API bool r_anal_set_reg_profile(RAnal *anal);
+R_API bool r_anal_set_reg_profile(RAnal *anal, const char *rp);
 R_API char *r_anal_get_reg_profile(RAnal *anal);
 R_API ut64 r_anal_get_bbaddr(RAnal *anal, ut64 addr);
 R_API bool r_anal_set_bits(RAnal *anal, int bits);
@@ -1561,7 +1568,7 @@ R_API const char *r_anal_stackop_tostring(int s);
 R_API RAnalOp *r_anal_op_new(void);
 R_API void r_anal_op_free(void *op);
 R_API void r_anal_op_init(RAnalOp *op);
-R_API bool r_anal_op_fini(RAnalOp *op);
+R_API void r_anal_op_fini(RAnalOp *op);
 R_API int r_anal_op_reg_delta(RAnal *anal, ut64 addr, const char *name);
 R_API bool r_anal_op_is_eob(RAnalOp *op);
 R_API RList *r_anal_op_list_new(void);
@@ -1572,6 +1579,8 @@ R_API char *r_anal_op_to_string(RAnal *anal, RAnalOp *op);
 R_API RAnalEsil *r_anal_esil_new(int stacksize, int iotrap, unsigned int addrsize);
 R_API bool r_anal_esil_set_pc(RAnalEsil *esil, ut64 addr);
 R_API bool r_anal_esil_setup(RAnalEsil *esil, RAnal *anal, int romem, int stats, int nonull);
+R_API void r_anal_esil_setup_macros(RAnalEsil *esil);
+R_API void r_anal_esil_setup_ops(RAnalEsil *esil);
 R_API void r_anal_esil_free(RAnalEsil *esil);
 R_API bool r_anal_esil_runword(RAnalEsil *esil, const char *word);
 R_API bool r_anal_esil_parse(RAnalEsil *esil, const char *str);
@@ -1639,7 +1648,7 @@ R_DEPRECATE R_API RAnalFunction *r_anal_get_fcn_in_bounds(RAnal *anal, ut64 addr
 
 R_API RAnalFunction *r_anal_get_function_byname(RAnal *anal, const char *name);
 
-R_API int r_anal_fcn(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int reftype);
+R_API int r_anal_function(RAnal *anal, RAnalFunction *fcn, ut64 addr, ut64 len, int reftype);
 R_API int r_anal_fcn_del(RAnal *anal, ut64 addr);
 R_API int r_anal_fcn_del_locs(RAnal *anal, ut64 addr);
 R_API bool r_anal_fcn_add_bb(RAnal *anal, RAnalFunction *fcn,
@@ -1688,10 +1697,10 @@ R_API RList *r_anal_xrefs_get_from(RAnal *anal, ut64 from);
 R_API void r_anal_xrefs_list(RAnal *anal, int rad);
 R_API RList *r_anal_function_get_refs(RAnalFunction *fcn);
 R_API RList *r_anal_function_get_xrefs(RAnalFunction *fcn);
-R_API int r_anal_xrefs_from(RAnal *anal, RList *list, const char *kind, const RAnalRefType type, ut64 addr);
-R_API int r_anal_xrefs_set(RAnal *anal, ut64 from, ut64 to, const RAnalRefType type);
-R_API int r_anal_xrefs_deln(RAnal *anal, ut64 from, ut64 to, const RAnalRefType type);
-R_API int r_anal_xref_del(RAnal *anal, ut64 at, ut64 addr);
+R_API bool r_anal_xrefs_from(RAnal *anal, RList *list, const char *kind, const RAnalRefType type, ut64 addr);
+R_API bool r_anal_xrefs_set(RAnal *anal, ut64 from, ut64 to, const RAnalRefType type);
+R_API bool r_anal_xrefs_deln(RAnal *anal, ut64 from, ut64 to, const RAnalRefType type);
+R_API bool r_anal_xref_del(RAnal *anal, ut64 at, ut64 addr);
 
 R_API RList *r_anal_get_fcns(RAnal *anal);
 
@@ -2130,6 +2139,8 @@ R_API RAnalEsilCFG *r_anal_esil_cfg_expr(RAnalEsilCFG *cfg, RAnal *anal, const u
 R_API RAnalEsilCFG *r_anal_esil_cfg_op(RAnalEsilCFG *cfg, RAnal *anal, RAnalOp *op);
 R_API void r_anal_esil_cfg_merge_blocks(RAnalEsilCFG *cfg);
 R_API void r_anal_esil_cfg_free(RAnalEsilCFG *cfg);
+R_API SdbGperf *r_anal_get_gperf_cc(const char *k);
+R_API SdbGperf *r_anal_get_gperf_types(const char *k);
 
 R_API RAnalEsilDFGNode *r_anal_esil_dfg_node_new(RAnalEsilDFG *edf, const char *c);
 R_API RAnalEsilDFG *r_anal_esil_dfg_new(RReg *regs);
@@ -2189,7 +2200,7 @@ extern RAnalPlugin r_anal_plugin_sh;
 extern RAnalPlugin r_anal_plugin_snes;
 extern RAnalPlugin r_anal_plugin_sparc_cs;
 extern RAnalPlugin r_anal_plugin_sparc_gnu;
-extern RAnalPlugin r_anal_plugin_sysz;
+extern RAnalPlugin r_anal_plugin_s390_cs;
 extern RAnalPlugin r_anal_plugin_tms320;
 extern RAnalPlugin r_anal_plugin_tms320c64x;
 extern RAnalPlugin r_anal_plugin_tricore;
@@ -2206,6 +2217,7 @@ extern RAnalPlugin r_anal_plugin_x86_udis;
 extern RAnalPlugin r_anal_plugin_xap;
 extern RAnalPlugin r_anal_plugin_xcore_cs;
 extern RAnalPlugin r_anal_plugin_xtensa;
+extern RAnalPlugin r_anal_plugin_arm_v35;
 extern RAnalPlugin r_anal_plugin_z80;
 extern RAnalPlugin r_anal_plugin_pyc;
 extern RAnalEsilPlugin r_esil_plugin_dummy;
