@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2009-2019 - pancake */
+/* radare - LGPL - Copyright 2009-2021 - pancake */
 
 #ifndef R2_CORE_H
 #define R2_CORE_H
@@ -30,13 +30,17 @@
 #include "r_util/r_print.h"
 #include "r_crypto.h"
 #include "r_bind.h"
+#include "r_codemeta.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 R_LIB_VERSION_HEADER(r_core);
 
+#define R_CORE_CMD_OK 0
+#define R_CORE_CMD_INVALID -1
 #define R_CORE_CMD_EXIT -2
+
 #define R_CORE_BLOCKSIZE 0x100
 #define R_CORE_BLOCKSIZE_MAX 0x3200000 /* 32MB */
 
@@ -52,6 +56,7 @@ R_LIB_VERSION_HEADER(r_core);
 #define R_FLAGS_FS_FUNCTIONS "functions"
 #define R_FLAGS_FS_IMPORTS "imports"
 #define R_FLAGS_FS_RELOCS "relocs"
+#define R_FLAGS_FS_REGISTERS "registers"
 #define R_FLAGS_FS_RESOURCES "resources"
 #define R_FLAGS_FS_SECTIONS "sections"
 #define R_FLAGS_FS_SEGMENTS "segments"
@@ -79,18 +84,16 @@ R_LIB_VERSION_HEADER(r_core);
 #define RTR_PROTOCOL_HTTP 3
 #define RTR_PROTOCOL_UNIX 4
 
-#define RTR_RAP_OPEN   0x01
-#define RTR_RAP_CMD    0x07
-#define RTR_RAP_REPLY  0x80
-
 #define RTR_MAX_HOSTS 255
 
 /* visual mode */
-#define R_CORE_VISUAL_MODE_PX  0
-#define R_CORE_VISUAL_MODE_PD  1
-#define R_CORE_VISUAL_MODE_DB  2
-#define R_CORE_VISUAL_MODE_OV  3
-#define R_CORE_VISUAL_MODE_CD  4
+typedef enum {
+	R_CORE_VISUAL_MODE_PX = 0,
+	R_CORE_VISUAL_MODE_PD = 1,
+	R_CORE_VISUAL_MODE_DB = 2,
+	R_CORE_VISUAL_MODE_OV = 3,
+	R_CORE_VISUAL_MODE_CD = 4
+} RCoreVisualMode;
 
 /*
 #define R_CORE_VISUAL_MODE_PC    4
@@ -101,6 +104,17 @@ R_LIB_VERSION_HEADER(r_core);
 #define R_CORE_VISUAL_MODE_PXR   9
 */
 
+typedef struct r_core_plugin_t {
+	const char *name;
+	const char *desc;
+	const char *license;
+	const char *author;
+	const char *version;
+	RCmdCb call; // returns true if command was handled, false otherwise.
+	RCmdCb init;
+	RCmdCb fini;
+} RCorePlugin;
+
 typedef struct r_core_rtr_host_t {
 	int proto;
 	char host[512];
@@ -133,30 +147,16 @@ typedef struct r_core_log_t {
 	RStrpool *sp;
 } RCoreLog;
 
-typedef struct r_core_file_t {
-	int dbg;
-	int fd;
-	RBinBind binb;
-	const struct r_core_t *core;
-	ut8 alive;
-} RCoreFile;
-
-
 typedef struct r_core_times_t {
 	ut64 loadlibs_init_time;
 	ut64 loadlibs_time;
 	ut64 file_open_time;
 } RCoreTimes;
 
-#define R_CORE_ASMSTEPS 128
 #define R_CORE_ASMQJMPS_NUM 10
 #define R_CORE_ASMQJMPS_LETTERS 26
 #define R_CORE_ASMQJMPS_MAX_LETTERS (26 * 26 * 26 * 26 * 26)
 #define R_CORE_ASMQJMPS_LEN_LETTERS 5
-typedef struct r_core_asmsteps_t {
-	ut64 offset;
-	int cols;
-} RCoreAsmsteps;
 
 typedef enum r_core_autocomplete_types_t {
 	R_CORE_AUTOCMPLT_DFLT = 0,
@@ -166,6 +166,7 @@ typedef enum r_core_autocomplete_types_t {
 	R_CORE_AUTOCMPLT_FCN,
 	R_CORE_AUTOCMPLT_ZIGN,
 	R_CORE_AUTOCMPLT_EVAL,
+	R_CORE_AUTOCMPLT_VARS,
 	R_CORE_AUTOCMPLT_PRJT,
 	R_CORE_AUTOCMPLT_MINS,
 	R_CORE_AUTOCMPLT_BRKP,
@@ -175,6 +176,7 @@ typedef enum r_core_autocomplete_types_t {
 	R_CORE_AUTOCMPLT_OPTN,
 	R_CORE_AUTOCMPLT_MS,
 	R_CORE_AUTOCMPLT_SDB,
+	R_CORE_AUTOCMPLT_CHRS,
 // --- left as last always
 	R_CORE_AUTOCMPLT_END,
 } RCoreAutocompleteType;
@@ -226,18 +228,45 @@ typedef struct {
 	char *cmd;
 } RCoreGadget;
 
-R_API void r_core_gadget_free (RCoreGadget *g);
+R_API void r_core_gadget_free(RCoreGadget *g);
 
-typedef struct r_core_t {
+typedef struct r_core_tasks_t {
+	int task_id_next;
+	RList *tasks;
+	RList *tasks_queue;
+	RList *oneshot_queue;
+	int oneshots_enqueued;
+	struct r_core_task_t *current_task;
+	struct r_core_task_t *main_task;
+	RThreadLock *lock;
+	int tasks_running;
+	bool oneshot_running;
+} RCoreTaskScheduler;
+
+typedef struct r_core_project_t {
+	char *name;
+	char *path;
+} RProject;
+
+R_API RProject *r_project_new(void);
+R_API bool r_project_rename(RProject *p, const char *newname);
+R_API bool r_project_is_git(RProject *p);
+R_API void r_project_close(RProject *p);
+R_API bool r_project_open(RProject *p, const char *prjname, const char *path);
+R_API void r_project_save(RProject *p);
+R_API void r_project_free(RProject *p);
+R_API bool r_project_is_loaded(RProject *p);
+R_API bool r_core_project_is_saved(RCore *core);
+
+struct r_core_t {
 	RBin *bin;
 	RConfig *config;
-	ut64 offset;
-	ut64 prompt_offset;
+	RProject *prj;
+	ut64 offset; // current seek
+	ut64 prompt_offset; // temporarily set to offset to have $$ in expressions always stay the same during temp seeks
 	ut32 blocksize;
 	ut32 blocksize_max;
 	ut8 *block;
-	ut8 *oobi; /* out of band input ; used for multiline or file input */
-	int oobi_len;
 	RBuffer *yank_buf;
 	ut64 yank_addr;
 	bool tmpseek;
@@ -246,8 +275,6 @@ typedef struct r_core_t {
 	/* files */
 	RCons *cons;
 	RIO *io;
-	RCoreFile *file;
-	RList *files;
 	RNum *num;
 	ut64 rc; // command's return code .. related to num->value;
 	RLib *lib;
@@ -255,7 +282,7 @@ typedef struct r_core_t {
 	RCmdDescriptor root_cmd_descriptor;
 	RList/*<RCmdDescriptor>*/ *cmd_descriptors;
 	RAnal *anal;
-	RAsm *assembler;
+	RAsm *rasm;
 	/* ^^ */
 	RCoreTimes *times;
 	RParse *parser;
@@ -271,18 +298,15 @@ typedef struct r_core_t {
 	RAGraph *graph;
 	RPanelsRoot *panels_root;
 	RPanels* panels;
-	char *cmdqueue;
+	RList *cmdqueue;
 	char *lastcmd;
 	char *cmdlog;
-	bool cfglog;
-	int cmdrepeat;
-	const char *cmdtimes;
-	bool cmd_in_backticks;
-	ut64 inc;
+	bool cfglog; // cfg.corelog
+	int cmdrepeat; // cmd.repeat
+	const char *cmdtimes; // cmd.times
+	R_DEPRECATE bool cmd_in_backticks; // whether currently executing a cmd out of backticks
 	int rtr_n;
 	RCoreRtrHost rtr_host[RTR_MAX_HOSTS];
-	int curasmstep;
-	RCoreAsmsteps asmsteps[R_CORE_ASMSTEPS];
 	ut64 *asmqjmps;
 	int asmqjmps_count;
 	int asmqjmps_size;
@@ -292,41 +316,31 @@ typedef struct r_core_t {
 	// visual // TODO: move them into RCoreVisual
 	int http_up;
 	int gdbserver_up;
-	int printidx;
+	RCoreVisualMode printidx;
 	char *stkcmd;
-	int vseek;
 	bool in_search;
 	RList *watchers;
 	RList *scriptstack;
-	int task_id_next;
-	RList *tasks;
-	RList *tasks_queue;
-	RList *oneshot_queue;
-	int oneshots_enqueued;
-	struct r_core_task_t *current_task;
-	struct r_core_task_t *main_task;
-	RThreadLock *tasks_lock;
-	int tasks_running;
-	bool oneshot_running;
+	RCoreTaskScheduler tasks;
 	int max_cmd_depth;
 	ut8 switch_file_view;
 	Sdb *sdb;
 	int incomment;
 	int curtab; // current tab
 	int seltab; // selected tab
-	int cmdremote;
+	char *cmdremote;
 	char *lastsearch;
 	char *cmdfilter;
 	bool break_loop;
-	RThreadLock *lock;
 	RList *undos;
 	bool binat;
-	bool fixedbits;
-	bool fixedarch;
+	bool fixedbits; // will be true when using @b:
+	bool fixedarch; // will be true when using @a:
 	bool fixedblock;
 	char *table_query;
 	int sync_index; // used for http.sync and T=
 	struct r_core_t *c2;
+	RTable *table;
 	RCoreAutocomplete *autocomplete;
 	int autocomplete_type;
 	int maxtab;
@@ -335,17 +349,21 @@ typedef struct r_core_t {
 	bool scr_gadgets;
 	bool log_events; // core.c:cb_event_handler : log actions from events if cfg.log.events is set
 	RList *ropchain;
+	char *theme;
+	int in_cmdstr;
+	bool marks_init;
+	ut64 marks[UT8_MAX + 1];
 
 	RMainCallback r_main_radare2;
 	// int (*r_main_radare2)(int argc, char **argv);
-	int (*r_main_rafind2)(int argc, char **argv);
-	int (*r_main_radiff2)(int argc, char **argv);
-	int (*r_main_rabin2)(int argc, char **argv);
-	int (*r_main_rarun2)(int argc, char **argv);
-	int (*r_main_ragg2)(int argc, char **argv);
-	int (*r_main_rasm2)(int argc, char **argv);
-	int (*r_main_rax2)(int argc, char **argv);
-} RCore;
+	int (*r_main_rafind2)(int argc, const char **argv);
+	int (*r_main_radiff2)(int argc, const char **argv);
+	int (*r_main_rabin2)(int argc, const char **argv);
+	int (*r_main_rarun2)(int argc, const char **argv);
+	int (*r_main_ragg2)(int argc, const char **argv);
+	int (*r_main_rasm2)(int argc, const char **argv);
+	int (*r_main_rax2)(int argc, const char **argv);
+};
 
 // maybe move into RAnal
 typedef struct r_core_item_t {
@@ -361,9 +379,8 @@ typedef struct r_core_item_t {
 	char *fcnname;
 } RCoreItem;
 
-
-R_API RCoreItem *r_core_item_at (RCore *core, ut64 addr);
-R_API void r_core_item_free (RCoreItem *ci);
+R_API RCoreItem *r_core_item_at(RCore *core, ut64 addr);
+R_API void r_core_item_free(RCoreItem *ci);
 
 R_API int r_core_bind(RCore *core, RCoreBind *bnd);
 
@@ -380,7 +397,7 @@ typedef int (*RCoreSearchCallback)(RCore *core, ut64 from, ut8 *buf, int len);
 #ifdef R_API
 //#define r_core_ncast(x) (RCore*)(size_t)(x)
 R_API RList *r_core_list_themes(RCore *core);
-R_API char *r_core_get_theme(void);
+R_API char *r_core_get_theme(RCore *core);
 R_API const char *r_core_get_section_name(RCore *core, ut64 addr);
 R_API RCons *r_core_get_cons(RCore *core);
 R_API RBin *r_core_get_bin(RCore *core);
@@ -388,11 +405,12 @@ R_API RConfig *r_core_get_config (RCore *core);
 R_API bool r_core_init(RCore *core);
 R_API void r_core_bind_cons(RCore *core); // to restore pointers in cons
 R_API RCore *r_core_new(void);
-R_API RCore *r_core_free(RCore *core);
-R_API RCore *r_core_fini(RCore *c);
+R_API void r_core_free(RCore *core);
+R_API void r_core_fini(RCore *c);
 R_API void r_core_wait(RCore *core);
 R_API RCore *r_core_ncast(ut64 p);
 R_API RCore *r_core_cast(void *p);
+R_API bool r_core_bin_load_structs(RCore *core, const char *file);
 R_API int r_core_config_init(RCore *core);
 R_API void r_core_config_update(RCore *core);
 R_API void r_core_parse_radare2rc(RCore *r);
@@ -400,26 +418,26 @@ R_API int r_core_prompt(RCore *core, int sync);
 R_API int r_core_prompt_exec(RCore *core);
 R_API int r_core_lines_initcache (RCore *core, ut64 start_addr, ut64 end_addr);
 R_API int r_core_lines_currline (RCore *core);
-R_API void r_core_prompt_loop(RCore *core);
+R_API bool r_core_prompt_loop(RCore *core);
 R_API ut64 r_core_pava(RCore *core, ut64 addr);
-R_API void run_pending_anal(RCore *core);
-R_API int r_core_cmd(RCore *core, const char *cmd, int log);
-R_API void r_core_cmd_repeat(RCore *core, int next);
+R_API int r_core_cmd(RCore *core, const char *cmd, bool log);
 R_API int r_core_cmd_task_sync(RCore *core, const char *cmd, bool log);
-R_API char *r_core_editor (const RCore *core, const char *file, const char *str);
+R_API char *r_core_editor(const RCore *core, const char *file, const char *str);
 R_API int r_core_fgets(char *buf, int len);
 R_API RFlagItem *r_core_flag_get_by_spaces(RFlag *f, ut64 off);
-R_API int r_core_cmdf(RCore *core, const char *fmt, ...);
-R_API int r_core_flush(RCore *core, const char *cmd);
+R_API int r_core_cmdf(RCore *core, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API int r_core_cmd0(RCore *core, const char *cmd);
+R_API void r_core_cmd_queue(RCore *core, const char *line);
+R_API void r_core_cmd_queue_wait(RCore *core);
 R_API void r_core_cmd_init(RCore *core);
 R_API int r_core_cmd_pipe(RCore *core, char *radare_cmd, char *shell_cmd);
 R_API char *r_core_cmd_str(RCore *core, const char *cmd);
-R_API char *r_core_cmd_strf(RCore *core, const char *fmt, ...);
+R_API char *r_core_cmd_strf(RCore *core, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API char *r_core_cmd_str_pipe(RCore *core, const char *cmd);
 R_API int r_core_cmd_file(RCore *core, const char *file);
 R_API int r_core_cmd_lines(RCore *core, const char *lines);
 R_API int r_core_cmd_command(RCore *core, const char *command);
+R_API void r_core_af(RCore *core, ut64 addr, const char *name, bool anal_calls);
 R_API bool r_core_run_script (RCore *core, const char *file);
 R_API bool r_core_seek(RCore *core, ut64 addr, bool rb);
 R_API bool r_core_visual_bit_editor(RCore *core);
@@ -427,7 +445,8 @@ R_API int r_core_seek_base (RCore *core, const char *hex);
 R_API void r_core_seek_previous (RCore *core, const char *type);
 R_API void r_core_seek_next (RCore *core, const char *type);
 R_API int r_core_seek_align(RCore *core, ut64 align, int count);
-R_API void r_core_seek_archbits (RCore *core, ut64 addr);
+R_API void r_core_arch_bits_at(RCore *core, ut64 addr, R_OUT R_NULLABLE int *bits, R_OUT R_BORROW R_NULLABLE const char **arch);
+R_API void r_core_seek_arch_bits(RCore *core, ut64 addr);
 R_API int r_core_block_read(RCore *core);
 R_API int r_core_block_size(RCore *core, int bsize);
 R_API int r_core_seek_size(RCore *core, ut64 addr, int bsize);
@@ -446,10 +465,10 @@ R_API bool r_core_prevop_addr(RCore* core, ut64 start_addr, int numinstrs, ut64*
 R_API ut64 r_core_prevop_addr_force(RCore *core, ut64 start_addr, int numinstrs);
 R_API bool r_core_visual_hudstuff(RCore *core);
 R_API int r_core_visual_classes(RCore *core);
+R_API int r_core_visual_anal_classes(RCore *core);
 R_API int r_core_visual_types(RCore *core);
 R_API int r_core_visual(RCore *core, const char *input);
 R_API int r_core_visual_graph(RCore *core, RAGraph *g, RAnalFunction *_fcn, int is_interactive);
-R_API bool r_core_visual_panels_root(RCore *core, RPanelsRoot *panels_root);
 R_API void r_core_visual_browse(RCore *core, const char *arg);
 R_API int r_core_visual_cmd(RCore *core, const char *arg);
 R_API void r_core_visual_seek_animation (RCore *core, ut64 addr);
@@ -468,15 +487,21 @@ R_API ut64 r_core_get_asmqjmps(RCore *core, const char *str);
 R_API void r_core_set_asmqjmps(RCore *core, char *str, size_t len, int i);
 R_API char* r_core_add_asmqjmp(RCore *core, ut64 addr);
 
+// core/panels.c
+R_API bool r_core_panels_root(RCore *core, RPanelsRoot *panels_root);
+R_API void r_core_panels_save(RCore *core, const char *_name);
+R_API bool r_core_panels_load(RCore *core, const char *_name);
+
 R_API void r_core_anal_type_init(RCore *core);
+R_API char *r_core_anal_hasrefs_to_depth(RCore *core, ut64 value, PJ *pj, int depth);
+R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn);
 R_API void r_core_anal_inflags (RCore *core, const char *glob);
-R_API int cmd_anal_objc (RCore *core, const char *input, bool auto_anal);
+R_API bool cmd_anal_objc (RCore *core, const char *input, bool auto_anal);
 R_API void r_core_anal_cc_init(RCore *core);
 R_API void r_core_anal_paths(RCore *core, ut64 from, ut64 to, bool followCalls, int followDepth, bool is_json);
-R_API void r_core_anal_esil_graph(RCore *core, const char *expr);
 
 R_API void r_core_list_io(RCore *core);
-R_API RListInfo *r_listinfo_new (char *name, RInterval pitv, RInterval vitv, int perm, char *extra);
+R_API RListInfo *r_listinfo_new (const char *name, RInterval pitv, RInterval vitv, int perm, const char *extra);
 R_API void r_listinfo_free (RListInfo *info);
 /* visual marks */
 R_API void r_core_visual_mark_seek(RCore *core, ut8 ch);
@@ -488,34 +513,24 @@ R_API void r_core_visual_mark_reset(RCore *core);
 
 R_API int r_core_search_cb(RCore *core, ut64 from, ut64 to, RCoreSearchCallback cb);
 R_API bool r_core_serve(RCore *core, RIODesc *fd);
-R_API int r_core_file_reopen(RCore *core, const char *args, int perm, int binload);
-R_API void r_core_file_reopen_debug(RCore *core, const char *args);
-R_API RCoreFile * r_core_file_find_by_fd(RCore* core, ut64 fd);
-R_API RCoreFile * r_core_file_find_by_name (RCore * core, const char * name);
-R_API RCoreFile * r_core_file_cur (RCore *r);
-R_API int r_core_file_set_by_fd(RCore *core, ut64 fd);
-R_API int r_core_file_set_by_name(RCore *core, const char * name);
-R_API int r_core_file_set_by_file (RCore * core, RCoreFile *cf);
-R_API int r_core_setup_debugger (RCore *r, const char *debugbackend, bool attach);
 
-R_API void r_core_file_free(RCoreFile *cf);
-R_API RCoreFile *r_core_file_open(RCore *core, const char *file, int flags, ut64 loadaddr);
-R_API RCoreFile *r_core_file_open_many(RCore *r, const char *file, int flags, ut64 loadaddr);
-R_API RCoreFile *r_core_file_get_by_fd(RCore *core, int fd);
-R_API int r_core_file_close(RCore *core, RCoreFile *fh);
-R_API bool r_core_file_close_fd(RCore *core, int fd);
+// RCoreFile APIs (bind, riodesc + rbinfile)
+R_API bool r_core_file_reopen(RCore *core, const char *args, int perm, int binload);
+R_API void r_core_file_reopen_debug(RCore *core, const char *args);
+R_API void r_core_file_reopen_remote_debug(RCore *core, char *uri, ut64 addr);
+R_API RIODesc *r_core_file_open(RCore *core, const char *file, int flags, ut64 loadaddr);
+R_API RIODesc *r_core_file_open_many(RCore *r, const char *file, int flags, ut64 loadaddr);
 R_API bool r_core_file_close_all_but(RCore *core);
-R_API int r_core_file_list(RCore *core, int mode);
-R_API int r_core_file_binlist(RCore *core);
-R_API bool r_core_file_bin_raise(RCore *core, ut32 num);
+
+
+R_API int r_core_setup_debugger (RCore *r, const char *debugbackend, bool attach);
 R_API int r_core_seek_delta(RCore *core, st64 addr);
-R_API int r_core_extend_at(RCore *core, ut64 addr, int size);
+R_API bool r_core_extend_at(RCore *core, ut64 addr, int size);
 R_API bool r_core_write_at(RCore *core, ut64 addr, const ut8 *buf, int size);
 R_API int r_core_write_op(RCore *core, const char *arg, char op);
 R_API ut8* r_core_transform_op(RCore *core, const char *arg, char op);
 R_API int r_core_set_file_by_fd (RCore * core, ut64 bin_fd);
 R_API int r_core_set_file_by_name (RBin * bin, const char * name);
-R_API ut32 r_core_file_cur_fd (RCore *core);
 
 R_API void r_core_debug_rr (RCore *core, RReg *reg, int mode);
 
@@ -525,8 +540,10 @@ R_API void r_core_fortune_list(RCore *core);
 R_API void r_core_fortune_print_random(RCore *core);
 
 /* project */
+#if 0
 R_API bool r_core_project_load(RCore *core, const char *prjfile, const char *rcfile);
 R_API RThread *r_core_project_load_bg(RCore *core, const char *prjfile, const char *rcfile);
+#endif
 R_API void r_core_project_execute_cmds(RCore *core, const char *prjfile);
 
 #define R_CORE_FOREIGN_ADDR -1
@@ -553,15 +570,15 @@ R_API int r_core_yank_file_all (RCore *core, const char *input);
 #define R_CORE_LOADLIBS_ALL UT32_MAX
 
 R_API void r_core_loadlibs_init(RCore *core);
-R_API int r_core_loadlibs(RCore *core, int where, const char *path);
+R_API bool r_core_loadlibs(RCore *core, int where, const char *path);
 R_API int r_core_cmd_buffer(RCore *core, const char *buf);
-R_API int r_core_cmdf(RCore *core, const char *fmt, ...);
+R_API int r_core_cmdf(RCore *core, const char *fmt, ...) R_PRINTF_CHECK(2, 3);
 R_API int r_core_cmd0(RCore *core, const char *cmd);
 R_API char *r_core_cmd_str(RCore *core, const char *cmd);
 R_API int r_core_cmd_foreach(RCore *core, const char *cmd, char *each);
 R_API int r_core_cmd_foreach3(RCore *core, const char *cmd, char *each);
 R_API char *r_core_op_str(RCore *core, ut64 addr);
-R_API RAnalOp *r_core_op_anal(RCore *core, ut64 addr);
+R_API RAnalOp *r_core_op_anal(RCore *core, ut64 addr, RAnalOpMask mask);
 R_API char *r_core_disassemble_instr(RCore *core, ut64 addr, int l);
 R_API char *r_core_disassemble_bytes(RCore *core, ut64 addr, int b);
 
@@ -569,44 +586,40 @@ R_API char *r_core_disassemble_bytes(RCore *core, ut64 addr, int b);
 R_API RList *r_core_get_func_args(RCore *core, const char *func_name);
 R_API void r_core_print_func_args(RCore *core);
 R_API char *resolve_fcn_name(RAnal *anal, const char * func_name);
-
 R_API int r_core_get_stacksz(RCore *core, ut64 from, ut64 to);
 
-/* cmd_type.c */
-R_API void r_core_link_stroff(RCore *core, RAnalFunction *fcn);
-
 /* anal.c */
 R_API RAnalOp* r_core_anal_op(RCore *core, ut64 addr, int mask);
+R_IPI int core_type_by_addr(RCore *core, ut64 addr);
 R_API void r_core_anal_esil(RCore *core, const char *str, const char *addr);
-R_API void r_core_anal_fcn_merge (RCore *core, ut64 addr, ut64 addr2);
+R_API void r_core_anal_fcn_merge(RCore *core, ut64 addr, ut64 addr2);
 R_API const char *r_core_anal_optype_colorfor(RCore *core, ut64 addr, bool verbose);
 R_API ut64 r_core_anal_address (RCore *core, ut64 addr);
-R_API void r_core_anal_undefine (RCore *core, ut64 off);
-R_API void r_core_anal_hint_print (RAnal* a, ut64 addr, int mode);
-R_API void r_core_anal_hint_list (RAnal *a, int mode);
+R_API void r_core_anal_undefine(RCore *core, ut64 off);
+R_API void r_core_anal_hint_print(RAnal* a, ut64 addr, int mode);
+R_API void r_core_anal_hint_list(RAnal *a, int mode);
 R_API int r_core_anal_search(RCore *core, ut64 from, ut64 to, ut64 ref, int mode);
-R_API int r_core_anal_search_xrefs(RCore *core, ut64 from, ut64 to, int rad);
-R_API int r_core_anal_data (RCore *core, ut64 addr, int count, int depth, int wordsize);
+R_API int r_core_anal_search_xrefs(RCore *core, ut64 from, ut64 to, PJ *pj, int rad);
+R_API int r_core_anal_data(RCore *core, ut64 addr, int count, int depth, int wordsize);
 R_API void r_core_anal_datarefs(RCore *core, ut64 addr);
 R_API void r_core_anal_coderefs(RCore *core, ut64 addr);
-R_API void r_core_anal_codexrefs(RCore *core, ut64 addr);
-R_API void r_core_anal_importxrefs(RCore *core);
+R_API RGraph/*<RGraphNodeInfo>*/ *r_core_anal_codexrefs(RCore *core, ut64 addr);
+R_API RGraph/*<RGraphNodeInfo>*/ *r_core_anal_importxrefs(RCore *core);
 R_API void r_core_anal_callgraph(RCore *core, ut64 addr, int fmt);
 R_API int r_core_anal_refs(RCore *core, const char *input);
 R_API void r_core_agraph_print(RCore *core, int use_utf, const char *input);
 R_API bool r_core_esil_cmd(RAnalEsil *esil, const char *cmd, ut64 a1, ut64 a2);
 R_API int r_core_esil_step(RCore *core, ut64 until_addr, const char *until_expr, ut64 *prev_addr, bool stepOver);
 R_API int r_core_esil_step_back(RCore *core);
-R_API int r_core_anal_bb(RCore *core, RAnalFunction *fcn, ut64 at, int head);
 R_API ut64 r_core_anal_get_bbaddr(RCore *core, ut64 addr);
 R_API bool r_core_anal_bb_seek(RCore *core, ut64 addr);
-R_API int r_core_anal_fcn(RCore *core, ut64 at, ut64 from, int reftype, int depth);
+R_API bool r_core_anal_fcn(RCore *core, ut64 at, ut64 from, int reftype, int depth);
 R_API char *r_core_anal_fcn_autoname(RCore *core, ut64 addr, int dump, int mode);
 R_API void r_core_anal_autoname_all_fcns(RCore *core);
 R_API void r_core_anal_autoname_all_golang_fcns(RCore *core);
 R_API int r_core_anal_fcn_list(RCore *core, const char *input, const char *rad);
 R_API char *r_core_anal_fcn_name(RCore *core, RAnalFunction *fcn);
-R_API int r_core_anal_fcn_list_size(RCore *core);
+R_API ut64 r_core_anal_fcn_list_size(RCore *core);
 R_API void r_core_anal_fcn_labels(RCore *core, RAnalFunction *fcn, int rad);
 R_API int r_core_anal_fcn_clean(RCore *core, ut64 addr);
 R_API int r_core_print_bb_custom(RCore *core, RAnalFunction *fcn);
@@ -621,9 +634,9 @@ R_API RList *r_core_anal_fcn_get_calls (RCore *core, RAnalFunction *fcn); // get
 
 /*tp.c*/
 R_API void r_core_anal_type_match(RCore *core, RAnalFunction *fcn);
-R_API RStrBuf *var_get_constraint (RAnal *a, RAnalVar *var);
 
 /* asm.c */
+#define R_MIDFLAGS_HIDE 0
 #define R_MIDFLAGS_SHOW 1
 #define R_MIDFLAGS_REALIGN 2
 #define R_MIDFLAGS_SYMALIGN 3
@@ -636,7 +649,7 @@ typedef struct r_core_asm_hit {
 } RCoreAsmHit;
 
 R_API RBuffer *r_core_syscall (RCore *core, const char *name, const char *args);
-R_API RBuffer *r_core_syscallf (RCore *core, const char *name, const char *fmt, ...);
+R_API RBuffer *r_core_syscallf (RCore *core, const char *name, const char *fmt, ...) R_PRINTF_CHECK(3, 4);
 R_API RCoreAsmHit *r_core_asm_hit_new(void);
 R_API RList *r_core_asm_hit_list_new(void);
 R_API void r_core_asm_hit_free(void *_hit);
@@ -647,17 +660,22 @@ R_API RList *r_core_asm_bwdisassemble (RCore *core, ut64 addr, int n, int len);
 R_API RList *r_core_asm_back_disassemble_instr (RCore *core, ut64 addr, int len, ut32 hit_count, ut32 extra_padding);
 R_API RList *r_core_asm_back_disassemble_byte (RCore *core, ut64 addr, int len, ut32 hit_count, ut32 extra_padding);
 R_API ut32 r_core_asm_bwdis_len (RCore* core, int* len, ut64* start_addr, ut32 l);
-R_API int r_core_print_disasm(RPrint *p, RCore *core, ut64 addr, ut8 *buf, int len, int lines, int invbreak, int nbytes, bool json, PJ *pj, RAnalFunction *pdf);
+R_API int r_core_print_disasm(RCore *core, ut64 addr, ut8 *buf, int len, int count, bool count_bytes, bool read_buffer_only, bool json, PJ *pj, RAnalFunction *pdf);
 R_API int r_core_print_disasm_json(RCore *core, ut64 addr, ut8 *buf, int len, int lines, PJ *pj);
-R_API int r_core_print_disasm_instructions (RCore *core, int len, int l);
-R_API int r_core_print_disasm_all (RCore *core, ut64 addr, int l, int len, int mode);
+R_API int r_core_print_disasm_instructions_with_buf(RCore *core, ut64 address, ut8 *buf, int nb_bytes, int nb_opcodes);
+R_API int r_core_print_disasm_instructions(RCore *core, int nb_bytes, int nb_opcodes);
+R_API int r_core_print_disasm_all(RCore *core, ut64 addr, int l, int len, int mode);
+R_API int r_core_disasm_pdi_with_buf(RCore *core, ut64 address, ut8 *buf, ut32 nb_opcodes, ut32 nb_bytes, int fmt);
 R_API int r_core_disasm_pdi(RCore *core, int nb_opcodes, int nb_bytes, int fmt);
+R_API int r_core_disasm_pde(RCore *core, int nb_opcodes, int mode);
 R_API int r_core_print_fcn_disasm(RPrint *p, RCore *core, ut64 addr, int l, int invbreak, int cbytes);
 R_API int r_core_get_prc_cols(RCore *core);
 R_API int r_core_flag_in_middle(RCore *core, ut64 at, int oplen, int *midflags);
 R_API int r_core_bb_starts_in_middle(RCore *core, ut64 at, int oplen);
 
+// both do the same, we should get rid of one of them
 R_API bool r_core_bin_raise (RCore *core, ut32 bfid);
+R_API bool r_core_bin_set_cur(RCore *core, RBinFile *binfile);
 
 R_API int r_core_bin_set_env (RCore *r, RBinFile *binfile);
 R_API int r_core_bin_set_by_fd (RCore *core, ut64 bin_fd);
@@ -665,14 +683,13 @@ R_API int r_core_bin_set_by_name (RCore *core, const char *name);
 R_API int r_core_bin_reload(RCore *core, const char *file, ut64 baseaddr);
 R_API bool r_core_bin_load(RCore *core, const char *file, ut64 baseaddr);
 R_API int r_core_bin_rebase(RCore *core, ut64 baddr);
-R_API void r_core_bin_export_info_rad(RCore *core);
+R_API void r_core_bin_export_info(RCore *core, int mode);
 R_API int r_core_bin_list(RCore *core, int mode);
 R_API bool r_core_bin_delete (RCore *core, ut32 binfile_idx);
 R_API ut64 r_core_bin_impaddr(RBin *bin, int va, const char *name);
 
 // XXX - this is kinda hacky, maybe there should be a way to
 // refresh the bin environment without specific calls?
-R_API int r_core_bin_refresh_strings(RCore *core);
 R_API int r_core_pseudo_code (RCore *core, const char *input);
 
 /* gdiff.c */
@@ -680,13 +697,13 @@ R_API int r_core_zdiff(RCore *c, RCore *c2);
 R_API int r_core_gdiff(RCore *core1, RCore *core2);
 R_API int r_core_gdiff_fcn(RCore *c, ut64 addr, ut64 addr2);
 
-R_API bool r_core_project_open(RCore *core, const char *file, bool thready);
+R_API bool r_core_project_open(RCore *core, const char *file);
 R_API int r_core_project_cat(RCore *core, const char *name);
 R_API int r_core_project_delete(RCore *core, const char *prjfile);
 R_API int r_core_project_list(RCore *core, int mode);
-R_API bool r_core_project_save_rdb(RCore *core, const char *file, int opts);
+R_API bool r_core_project_save_script(RCore *core, const char *file, int opts);
 R_API bool r_core_project_save(RCore *core, const char *file);
-R_API char *r_core_project_info(RCore *core, const char *file);
+R_API char *r_core_project_name(RCore *core, const char *file);
 R_API char *r_core_project_notes_file (RCore *core, const char *file);
 
 R_API char *r_core_sysenv_begin(RCore *core, const char *cmd);
@@ -721,6 +738,7 @@ R_API void r_core_recover_vars(RCore *core, RAnalFunction *fcn, bool argonly);
 #define R_CORE_BIN_ACC_SOURCE 0x800000
 #define R_CORE_BIN_ACC_HASHES 0x10000000
 #define R_CORE_BIN_ACC_TRYCATCH 0x20000000
+#define R_CORE_BIN_ACC_SECTIONS_MAPPING 0x40000000
 #define R_CORE_BIN_ACC_ALL	0x504FFF
 
 #define R_CORE_PRJ_FLAGS	0x0001
@@ -742,11 +760,11 @@ typedef struct r_core_bin_filter_t {
 	const char *name;
 } RCoreBinFilter;
 
-R_API int r_core_bin_info (RCore *core, int action, int mode, int va, RCoreBinFilter *filter, const char *chksum);
+R_API int r_core_bin_info (RCore *core, int action, PJ *pj, int mode, int va, RCoreBinFilter *filter, const char *chksum);
 R_API int r_core_bin_set_arch_bits (RCore *r, const char *name, const char * arch, ut16 bits);
 R_API int r_core_bin_update_arch_bits (RCore *r);
 R_API char *r_core_bin_method_flags_str(ut64 flags, int mode);
-R_API int r_core_pdb_info(RCore *core, const char *file, ut64 baddr, int mode);
+R_API bool r_core_pdb_info(RCore *core, const char *file, PJ *pj, int mode);
 
 /* rtr */
 R_API int r_core_rtr_cmds (RCore *core, const char *port);
@@ -757,6 +775,7 @@ R_API void r_core_rtr_list(RCore *core);
 R_API void r_core_rtr_add(RCore *core, const char *input);
 R_API void r_core_rtr_remove(RCore *core, const char *input);
 R_API void r_core_rtr_session(RCore *core, const char *input);
+R_API void r_core_rtr_event(RCore *core, const char *input);
 R_API void r_core_rtr_cmd(RCore *core, const char *input);
 R_API int r_core_rtr_http(RCore *core, int launch, int browse, const char *path);
 R_API int r_core_rtr_http_stop(RCore *u);
@@ -776,7 +795,7 @@ R_API int r_core_visual_view_zigns(RCore *core);
 R_API int r_core_visual_view_rop(RCore *core);
 R_API int r_core_visual_comments(RCore *core);
 R_API int r_core_visual_prompt(RCore *core);
-R_API bool r_core_visual_esil (RCore *core);
+R_API bool r_core_visual_esil (RCore *core, const char *input);
 R_API int r_core_search_preludes(RCore *core, bool log);
 R_API int r_core_search_prelude(RCore *core, ut64 from, ut64 to, const ut8 *buf, int blen, const ut8 *mask, int mlen);
 R_API RList* /*<RIOMap*>*/ r_core_get_boundaries_prot (RCore *core, int protection, const char *mode, const char *prefix);
@@ -819,6 +838,7 @@ R_API void r_core_log_del(RCore *core, int n);
 // TODO MOVE SOMEWHERE ELSE
 typedef char *(*PrintItemCallback)(void *user, void *p, bool selected);
 R_API char *r_str_widget_list(void *user, RList *list, int rows, int cur, PrintItemCallback cb);
+R_API PJ *r_core_pj_new(RCore *core);
 /* help */
 R_API void r_core_cmd_help(const RCore *core, const char * help[]);
 
@@ -841,7 +861,7 @@ typedef struct {
 
 R_API bool core_anal_bbs(RCore *core, const char* input);
 R_API bool core_anal_bbs_range (RCore *core, const char* input);
-R_API char *r_core_anal_hasrefs(RCore *core, ut64 value, bool verbose);
+R_API char *r_core_anal_hasrefs(RCore *core, ut64 value, int mode);
 R_API char *r_core_anal_get_comments(RCore *core, ut64 addr);
 R_API RCoreAnalStats* r_core_anal_get_stats (RCore *a, ut64 from, ut64 to, ut64 step);
 R_API void r_core_anal_stats_free (RCoreAnalStats *s);
@@ -855,6 +875,9 @@ R_API int r_line_hist_offset_down(RLine *line);
 
 // TODO : move into debug or syscall++
 R_API char *cmd_syscall_dostr(RCore *core, st64 num, ut64 addr);
+R_API void cmd_agfb(RCore *core);
+R_API void cmd_agfb2(RCore *core, const char *s);
+R_API void cmd_agfb3(RCore *core, const char *s, int x, int y);
 
 /* tasks */
 
@@ -889,32 +912,34 @@ typedef struct r_core_task_t {
 typedef void (*RCoreTaskOneShot)(void *);
 
 R_API void r_core_echo(RCore *core, const char *msg);
-R_API RCoreTask *r_core_task_get(RCore *core, int id);
-R_API RCoreTask *r_core_task_get_incref(RCore *core, int id);
-R_API RTable *r_core_table(RCore *core);
-R_API void r_core_task_print(RCore *core, RCoreTask *task, int mode);
+R_API RTable *r_core_table(RCore *core, const char *name);
+
+R_API void r_core_task_scheduler_init (RCoreTaskScheduler *tasks, RCore *core);
+R_API void r_core_task_scheduler_fini (RCoreTaskScheduler *tasks);
+R_API RCoreTask *r_core_task_get(RCoreTaskScheduler *scheduler, int id);
+R_API RCoreTask *r_core_task_get_incref(RCoreTaskScheduler *scheduler, int id);
+R_API void r_core_task_print(RCore *core, RCoreTask *task, PJ *pj, int mode);
 R_API void r_core_task_list(RCore *core, int mode);
-R_API int r_core_task_running_tasks_count(RCore *core);
+R_API int r_core_task_running_tasks_count(RCoreTaskScheduler *scheduler);
 R_API const char *r_core_task_status(RCoreTask *task);
 R_API RCoreTask *r_core_task_new(RCore *core, bool create_cons, const char *cmd, RCoreTaskCallback cb, void *user);
 R_API void r_core_task_incref(RCoreTask *task);
 R_API void r_core_task_decref(RCoreTask *task);
-R_API void r_core_task_enqueue(RCore *core, RCoreTask *task);
-R_API void r_core_task_enqueue_oneshot(RCore *core, RCoreTaskOneShot func, void *user);
-R_API int r_core_task_run_sync(RCore *core, RCoreTask *task);
-R_API void r_core_task_sync_begin(RCore *core);
-R_API void r_core_task_sync_end(RCore *core);
-R_API void r_core_task_continue(RCoreTask *t);
+R_API void r_core_task_enqueue(RCoreTaskScheduler *scheduler, RCoreTask *task);
+R_API void r_core_task_enqueue_oneshot(RCoreTaskScheduler *scheduler, RCoreTaskOneShot func, void *user);
+R_API int r_core_task_run_sync(RCoreTaskScheduler *scheduler, RCoreTask *task);
+R_API void r_core_task_sync_begin(RCoreTaskScheduler *scheduler);
+R_API void r_core_task_sync_end(RCoreTaskScheduler *scheduler);
+R_API void r_core_task_yield(RCoreTaskScheduler *scheduler);
 R_API void r_core_task_sleep_begin(RCoreTask *task);
 R_API void r_core_task_sleep_end(RCoreTask *task);
-R_API void r_core_task_break(RCore *core, int id);
-R_API void r_core_task_break_all(RCore *core);
-R_API int r_core_task_del(RCore *core, int id);
-R_API void r_core_task_del_all_done(RCore *core);
-R_API RCoreTask *r_core_task_self(RCore *core);
-R_API void r_core_task_join(RCore *core, RCoreTask *current, int id);
-typedef void (*inRangeCb) (RCore *core, ut64 from, ut64 to, int vsize,
-		int count, void *cb_user);
+R_API void r_core_task_break(RCoreTaskScheduler *scheduler, int id);
+R_API void r_core_task_break_all(RCoreTaskScheduler *scheduler);
+R_API int r_core_task_del(RCoreTaskScheduler *scheduler, int id);
+R_API void r_core_task_del_all_done(RCoreTaskScheduler *scheduler);
+R_API RCoreTask *r_core_task_self(RCoreTaskScheduler *scheduler);
+R_API void r_core_task_join(RCoreTaskScheduler *scheduler, RCoreTask *current, int id);
+typedef void (*inRangeCb) (RCore *core, ut64 from, ut64 to, int vsize, void *cb_user);
 R_API int r_core_search_value_in_range (RCore *core, RInterval search_itv,
 		ut64 vmin, ut64 vmax, int vsize, inRangeCb cb, void *cb_user);
 
@@ -923,11 +948,49 @@ R_API void r_core_autocomplete_free(RCoreAutocomplete *obj);
 R_API void r_core_autocomplete_reload (RCore *core);
 R_API RCoreAutocomplete *r_core_autocomplete_find(RCoreAutocomplete *parent, const char* cmd, bool exact);
 R_API bool r_core_autocomplete_remove(RCoreAutocomplete *parent, const char* cmd);
-R_API void r_core_anal_propagate_noreturn(RCore *core);
+R_API void r_core_anal_propagate_noreturn(RCore *core, ut64 addr);
 
 /* PLUGINS */
 extern RCorePlugin r_core_plugin_java;
 extern RCorePlugin r_core_plugin_a2f;
+extern RCorePlugin r_core_plugin_sixref;
+R_API bool r_core_plugin_init(RCmd *cmd);
+R_API bool r_core_plugin_add(RCmd *cmd, RCorePlugin *plugin);
+R_API bool r_core_plugin_check(RCmd *cmd, const char *a0);
+R_API bool r_core_plugin_fini(RCmd *cmd);
+
+
+/* DECOMPILER PRINTING FUNCTIONS */
+/**
+ * @brief Prints the data contained in the specified RAnnotatedCode in JSON format.
+ *
+ * The function will print the output in console using the function r_cons_printf();
+ *
+ * @param code Pointer to a RAnnotatedCode.
+ */
+R_API void r_codemeta_print_json(RCodeMeta *code);
+/**
+ * @brief Prints the decompiled code from the specified RAnnotatedCode.
+ *
+ * This function is used for printing the output of commands pdg and pdgo.
+ * It can print the decompiled code with or without offsets. If line_offsets is a null pointer,
+ * the output will be printed without offsets (pdg), otherwise, the output will be
+ * printed with offsets.
+ * This function will print the output in console using the function r_cons_printf();
+ *
+ * @param code Pointer to a RAnnotatedCode.
+ * @param line_offsets Pointer to a @ref RVector that contains offsets for the decompiled code.
+ */
+R_API void r_codemeta_print(RCodeMeta *code, RVector *line_offsets);
+/**
+ * @brief  Prints the decompiled code as comments
+ *
+ * This function is used for the output of command pdg*
+ * Output will be printed in console using the function r_cons_printf();
+ *
+ * @param code Pointer to a RAnnotatedCode.
+ */
+R_API void r_codemeta_print_comment_cmds(RCodeMeta *code);
 
 #endif
 
